@@ -14,16 +14,16 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with tensap.  If not, see <https://www.gnu.org/licenses/>.
 
-'''
+"""
 Module dimension_tree.
 
-'''
+"""
 
 import numpy as np
 
 
 class DimensionTree:
-    '''
+    """
     Class DimensionTree.
 
     Attributes
@@ -65,10 +65,10 @@ class DimensionTree:
     plot_options : dict
         Options for plotting the tree.
 
-    '''
+    """
 
     def __init__(self, dim2ind, adjacency_matrix):
-        '''
+        """
         Constructor of the class DimensionTree.
 
         Create a dimension partition tree over D = {1,...,d} from an
@@ -87,27 +87,25 @@ class DimensionTree:
         -------
         None.
 
-        '''
+        """
         self.dim2ind = np.array(dim2ind)
         self.adjacency_matrix = np.array(adjacency_matrix)
-        self.plot_options = {'level_alignment': False}
+        self.plot_options = {"level_alignment": False}
 
         self._precompute_attributes()
         self.update_dims_from_leaves()
 
     def __repr__(self):
-        return ('<DimensionTree:{n}' +
-                '{t}nb_nodes = {},{n}' +
-                '{t}arity = {},{n}' +
-                '{t}dim2ind = {},{n}' +
-                '{t}is_leaf = {},{n}').format(self.nb_nodes,
-                                              self.arity,
-                                              self.dim2ind,
-                                              self.is_leaf,
-                                              t='\t', n='\n')
+        return (
+            "<DimensionTree:{n}"
+            + "{t}nb_nodes = {},{n}"
+            + "{t}arity = {},{n}"
+            + "{t}dim2ind = {},{n}"
+            + "{t}is_leaf = {},{n}"
+        ).format(self.nb_nodes, self.arity, self.dim2ind, self.is_leaf, t="\t", n="\n")
 
     def permute(self, sigma):
-        '''
+        """
         Permute the dimensions of the DimensionTree.
 
         Parameters
@@ -120,13 +118,13 @@ class DimensionTree:
         DimensionTree
             A DimensionTree with permuted dimensions.
 
-        '''
+        """
         dim2ind = np.copy(self.dim2ind)
         dim2ind[sigma] = dim2ind.flatten()
         return DimensionTree(dim2ind, np.copy(self.adjacency_matrix))
 
     def ipermute(self, sigma):
-        '''
+        """
         Inverse permutation of the dimensions of the DimensionTree.
 
         Parameters
@@ -139,28 +137,29 @@ class DimensionTree:
         DimensionTree
             A DimensionTree with permuted dimensions.
 
-        '''
+        """
         dim2ind = np.copy(self.dim2ind)
         dim2ind = dim2ind[sigma].flatten()
         return DimensionTree(dim2ind, np.copy(self.adjacency_matrix))
 
     def __eq__(self, T):
-        is_equal = (len(self.dim2ind) == len(T.dim2ind)) and \
-            (self.nb_nodes == T.nb_nodes) and \
-            (np.max(self.level) == np.max(T.level))
+        is_equal = (
+            (len(self.dim2ind) == len(T.dim2ind))
+            and (self.nb_nodes == T.nb_nodes)
+            and (np.max(self.level) == np.max(T.level))
+        )
 
         if is_equal:
-            for level in np.arange(1, np.max(self.level)+1):
+            for level in np.arange(1, np.max(self.level) + 1):
                 nodes1 = self.nodes_with_level(level)
                 nodes2 = T.nodes_with_level(level)
                 if len(nodes1) != len(nodes2):
                     return is_equal
-                dims1 = [self.dims[i] for i in nodes1-1]
-                dims2 = [T.dims[i] for i in nodes2-1]
+                dims1 = [self.dims[i] for i in nodes1 - 1]
+                dims2 = [T.dims[i] for i in nodes2 - 1]
                 for i, dim1 in enumerate(dims1):
                     for dim2 in dims2:
-                        if np.array_equal(np.sort(dim1),
-                                          np.sort(dim2)):
+                        if np.array_equal(np.sort(dim1), np.sort(dim2)):
                             nodes1[i] = 0
                             break
                 if any(nodes1 != 0):
@@ -172,7 +171,7 @@ class DimensionTree:
         return not self == T
 
     def children(self, nod):
-        '''
+        """
         Return the children of a given node.
 
         Parameters
@@ -185,12 +184,12 @@ class DimensionTree:
         list
             List of children of nod.
 
-        '''
-        children = self._children[:, nod-1]
+        """
+        children = self._children[:, nod - 1]
         return children[children != 0]
 
     def child_number(self, nod):
-        '''
+        """
         Return the child number of nod.
 
         Parameters
@@ -203,11 +202,11 @@ class DimensionTree:
         int
             The child number.
 
-        '''
-        return self._child_number[nod-1]
+        """
+        return self._child_number[nod - 1]
 
     def parent(self, nod):
-        '''
+        """
         Return the parent of a given node.
 
         Parameters
@@ -220,11 +219,11 @@ class DimensionTree:
         int
             The parent of nod.
 
-        '''
-        return self._parent[nod-1]
+        """
+        return self._parent[nod - 1]
 
     def ascendants(self, nod):
-        '''
+        """
         Return the ascendants of a given node.
 
         Parameters
@@ -237,7 +236,7 @@ class DimensionTree:
         anod : list
             List of ascendants of nod.
 
-        '''
+        """
         anod = []
         pnod = self.parent(nod)
         while pnod:
@@ -246,7 +245,7 @@ class DimensionTree:
         return np.array(anod)
 
     def descendants(self, nod):
-        '''
+        """
         Return the descendants of a given node.
 
         Parameters
@@ -259,18 +258,109 @@ class DimensionTree:
         dnod : list
             List of descendants of nod.
 
-        '''
+        """
         dnod = np.array([], dtype=int)
         chnod = self.children(nod)
         chnod = chnod[chnod != 0]
         if len(chnod) != 0:
             for children in chnod:
-                dnod = np.concatenate((dnod, [children],
-                                       self.descendants(children)))
+                dnod = np.concatenate((dnod, [children], self.descendants(children)))
         return np.array(dnod)
 
+    def change_root(self, nod):
+        """
+        Change the root node of a dimension tree T.
+
+        The function returns a new dimension tree whose root is the node nod,
+        along with the modified nodes in the tree T
+
+        If nod is a leaf node of T, a new node is created, associated with
+        dimension T.dims{nod}
+
+
+        Parameters
+        ----------
+        nod : int
+            Number of the node which becomes the new root
+
+        Returns
+        -------
+        Tnew : DimensionTree
+            New dimension tree with root nod
+        mnodes : list
+            List of modified nodes.
+
+        """
+
+        if nod == self.root:
+            return self, np.array([])
+
+        A = np.copy(self.adjacency_matrix)
+
+        a = self.ascendants(nod)
+        a = np.concatenate(([nod], a))
+
+        chnod = self.children(nod)
+        modifiedNodes = np.concatenate(([nod], chnod))
+        for i in np.arange(a.size - 1):
+            A[a[i] - 1, a[i + 1] - 1] = 1
+            A[a[i + 1] - 1, a[i] - 1] = 0
+            chnod = self.children(a[i + 1])
+            modifiedNodes = np.concatenate((modifiedNodes, [a[i + 1]], chnod))
+        modifiedNodes = np.unique(modifiedNodes)
+        if self.is_leaf[nod - 1]:
+            n = np.copy(self.nb_nodes)
+            Anew = np.zeros([n + 1] * 2, dtype=int)
+            Anew[:n, :n] = A
+            Anew[nod - 1, n] = 1
+            dim2ind = np.copy(self.dim2ind)
+            dim2ind[self.dims[nod - 1]] = n + 1
+        else:
+            Anew = A
+            dim2ind = np.copy(self.dim2ind)
+
+        return DimensionTree(dim2ind, Anew), modifiedNodes
+
+    def add_child(self, nod):
+        """
+        Add a child node to the node nod of a dimension tree T and returns
+        a new dimension tree.
+
+        If nod is not leaf node, the new node is associated
+        with a new dimension length(T.dim2ind)+1
+
+        If nod is a leaf node, the new node is associated
+        with the dimension of node nod
+
+
+        Parameters
+        ----------
+        nod : int
+            Number of the node to which we add a child
+
+        Returns
+        -------
+        Tnew : DimensionTree
+            New dimension tree
+
+        """
+
+        A = np.copy(self.adjacency_matrix)
+        n = self.nb_nodes
+        Anew = np.zeros([n + 1] * 2, dtype=int)
+        Anew[:n, :n] = A
+        Anew[nod - 1, n] = 1
+        if self.is_leaf[nod - 1]:
+            dim2ind = np.copy(self.dim2ind)
+            dim2ind[dim2ind == nod] = n + 1
+        else:
+            dim2ind = np.copy(self.dim2ind)
+            dim2ind = np.concatenate((dim2ind, [n + 1]))
+
+        return DimensionTree(dim2ind, Anew)
+
     def sub_dimension_tree(self, root):
-        '''
+        """
         Extract a sub dimension tree.
 
         The attribute dim2ind of the sub dimension tree gives the nodes indices
@@ -288,15 +378,15 @@ class DimensionTree:
         nod : np.array
             Extracted nodes from T.
 
-        '''
-        dims = self.dims[root-1]
+        """
+        dims = self.dims[root - 1]
         nod = np.concatenate(([root], self.descendants(root)))
-        adj_mat = np.copy(self.adjacency_matrix[nod[:, None]-1, nod-1])
+        adj_mat = np.copy(self.adjacency_matrix[nod[:, None] - 1, nod - 1])
         dim2ind = np.nonzero(np.isin(nod, self.dim2ind[dims]))[0] + 1
         return DimensionTree(dim2ind, adj_mat), nod
 
     def node_with_dims(self, dims):
-        '''
+        """
         Return the index of the node with given set of dimensions.
 
         Return the index of the node corresponding to dimensions dims or an
@@ -312,13 +402,13 @@ class DimensionTree:
         numpy.ndarray
             Index of the node with the given set of dimensions dims.
 
-        '''
+        """
         dims = np.sort(dims)
         nod = [np.array_equal(dims, np.sort(x)) for x in self.dims]
-        return np.nonzero(nod)[0]+1
+        return np.nonzero(nod)[0] + 1
 
     def update_dims_from_leaves(self):
-        '''
+        """
         Update the dimensions of all nodes from the dimensions of the leaves
         given in T.dim2ind.
 
@@ -327,24 +417,25 @@ class DimensionTree:
         DimensionTree
             The DimensionTree object with updated attribute dims.
 
-        '''
+        """
         _dims = np.zeros(self.nb_nodes, dtype=int)
-        _dims[self.dim2ind-1] = np.arange(len(self.dim2ind))
+        _dims[self.dim2ind - 1] = np.arange(len(self.dim2ind))
         _dims = np.split(_dims, len(_dims))
 
         for level in np.arange(np.max(self.level), -1, -1):
             nod_level = self.nodes_with_level(level)
             for nod in nod_level:
-                if not self.is_leaf[nod-1]:
-                    children = self._children[:, nod-1]
-                    _dims[nod-1] = np.hstack([_dims[i-1] for i in
-                                              children[children != 0]])
+                if not self.is_leaf[nod - 1]:
+                    children = self._children[:, nod - 1]
+                    _dims[nod - 1] = np.hstack(
+                        [_dims[i - 1] for i in children[children != 0]]
+                    )
 
         self.dims = _dims
         return self
 
     def nodes_with_level(self, level):
-        '''
+        """
         Return the indices of the nodes at a given level.
 
         Parameters
@@ -357,13 +448,13 @@ class DimensionTree:
         numpy.ndarray
             Nodes with given level.
 
-        '''
+        """
         ind_lvl = np.repeat(False, self.nb_nodes)
         ind_lvl[self.level == level] = True
-        return np.nonzero(ind_lvl)[0]+1
+        return np.nonzero(ind_lvl)[0] + 1
 
     def tree_layout(self):
-        '''
+        """
         Return the layout of a tree.
 
         Returns
@@ -371,9 +462,10 @@ class DimensionTree:
         dict
             The layout of the tree, used for plotting.
 
-        '''
+        """
+
         def post_order(T, root=None):
-            '''
+            """
             Return the post-ordering of a tree.
 
             Parameters
@@ -389,7 +481,7 @@ class DimensionTree:
             post : numpy.ndarray
                 The post-ordering of the tree or of one of its subtrees.
 
-            '''
+            """
             if root is None:
                 root = T.root
             post = np.array([], dtype=int)
@@ -408,32 +500,33 @@ class DimensionTree:
         y = np.zeros(self.nb_nodes)
         n_leaves = 0
         for nod in post:
-            if self.is_leaf[nod-1]:
+            if self.is_leaf[nod - 1]:
                 n_leaves += 1
-                xmin[nod-1] = n_leaves
-                xmax[nod-1] = n_leaves
-                y[nod-1] = 0
+                xmin[nod - 1] = n_leaves
+                xmax[nod - 1] = n_leaves
+                y[nod - 1] = 0
             pa = self.parent(nod)
-            xmin[pa-1] = np.min((xmin[pa-1], xmin[nod-1]))
-            xmax[pa-1] = np.max((xmax[pa-1], xmax[nod-1]))
-            y[pa-1] = np.max((y[pa-1], y[nod-1]+1))
+            xmin[pa - 1] = np.min((xmin[pa - 1], xmin[nod - 1]))
+            xmax[pa - 1] = np.max((xmax[pa - 1], xmax[nod - 1]))
+            y[pa - 1] = np.max((y[pa - 1], y[nod - 1] + 1))
 
         x = (xmin + xmax) / 2
 
-        x /= (self.dim2ind.size + 1)
-        y = (y+1) / (np.max(y)+2)
+        x /= self.dim2ind.size + 1
+        y = (y + 1) / (np.max(y) + 2)
 
-        if self.plot_options['level_alignment']:
+        if self.plot_options["level_alignment"]:
             height = np.max(y) - np.min(y)
-            for alpha in np.arange(1, self.nb_nodes+1):
-                y[alpha-1] = np.max(y) - self.level[alpha-1] * \
-                    height / np.max(self.level)
+            for alpha in np.arange(1, self.nb_nodes + 1):
+                y[alpha - 1] = np.max(y) - self.level[alpha - 1] * height / np.max(
+                    self.level
+                )
 
         pos = [np.array([xx, yy]) for xx, yy in zip(x, y)]
         return dict(zip(range(self.nb_nodes), pos))
 
     def plot(self, **args):
-        '''
+        """
         Plot the tree with the nodes indices.
 
         This method requires the package igraph.
@@ -449,12 +542,13 @@ class DimensionTree:
         -------
         None.
 
-        '''
+        """
         self.plot_with_labels_at_nodes(self.nodes_indices, **args)
 
-    def plot_with_labels_at_nodes(self, labels, node_color='red',
-                                  colored_nodes=None, title=None):
-        '''
+    def plot_with_labels_at_nodes(
+        self, labels, node_color="red", colored_nodes=None, title=None
+    ):
+        """
         Plot the tree with labels at nodes.
 
         This method requires the package igraph.
@@ -474,50 +568,55 @@ class DimensionTree:
         -------
         None.
 
-        '''
+        """
         import networkx as nx
         import matplotlib.pyplot as plt
 
         plt.figure()
-        g = nx.convert_matrix.from_numpy_matrix(self.adjacency_matrix)
+        g = nx.convert_matrix.from_numpy_array(self.adjacency_matrix)
         pos = self.tree_layout()
-        nx.draw_networkx_nodes(g, pos,
-                               nodelist=np.setdiff1d(
-                                   np.arange(1, self.nb_nodes+1),
-                                   colored_nodes)-1,
-                               node_color='w',
-                               edgecolors='k',
-                               node_size=100)
+        nx.draw_networkx_nodes(
+            g,
+            pos,
+            nodelist=np.setdiff1d(np.arange(1, self.nb_nodes + 1), colored_nodes) - 1,
+            node_color="w",
+            edgecolors="k",
+            node_size=100,
+        )
 
         if colored_nodes is not None:
-            nx.draw_networkx_nodes(g, pos,
-                                   nodelist=colored_nodes-1,
-                                   node_color=node_color,
-                                   edgecolors='k',
-                                   node_size=100)
+            nx.draw_networkx_nodes(
+                g,
+                pos,
+                nodelist=colored_nodes - 1,
+                node_color=node_color,
+                edgecolors="k",
+                node_size=100,
+            )
 
         nx.draw_networkx_edges(g, pos)
-        labels = dict(zip(range(self.nb_nodes), [x if x is not None
-                                                 else '' for x in labels]))
+        labels = dict(
+            zip(range(self.nb_nodes), [x if x is not None else "" for x in labels])
+        )
         y = [x[1] for x in pos.values()]
         min_height = np.min(y)
-        shift = (np.max(y)-np.min(y))/np.max(self.level)/2
+        shift = (np.max(y) - np.min(y)) / np.max(self.level) / 2
         pos_labels = {}
         for k, v in pos.items():
             if v[1] == min_height:
-                pos_labels[k] = (v[0], v[1]-shift)
+                pos_labels[k] = (v[0], v[1] - shift)
             else:
-                pos_labels[k] = (v[0], v[1]+shift)
+                pos_labels[k] = (v[0], v[1] + shift)
         nx.draw_networkx_labels(g, pos_labels, labels)
 
-        plt.axis('off')
+        plt.axis("off")
         plt.title(title)
         axes = plt.gca()
         axes.set_ylim([0, 1])
         plt.show()
 
     def plot_dims(self, nodes=None, **args):
-        '''
+        """
         Plot the dimensions associated with the nodes of the tree.
 
         This method requires the package igraph.
@@ -532,15 +631,17 @@ class DimensionTree:
         -------
         None.
 
-        '''
+        """
         if nodes is None:
             nodes = self.dim2ind
-        labels = [np.nonzero(nodes == i+1)[0][0] if self.is_leaf[i] else None
-                  for i in range(self.nb_nodes)]
+        labels = [
+            np.nonzero(nodes == i + 1)[0][0] if self.is_leaf[i] else None
+            for i in range(self.nb_nodes)
+        ]
         self.plot_with_labels_at_nodes(labels=labels, **args)
 
     def _precompute_attributes(self):
-        '''
+        """
         Precompute the attributes of the DimensionTree from the attributes
         adjacency_matrix and dim2ind.
 
@@ -549,55 +650,54 @@ class DimensionTree:
         DimensionTree
             A DimensionTree with updated attributes.
 
-        '''
+        """
         self.nb_nodes = self.adjacency_matrix.shape[0]
-        self.nodes_indices = np.arange(1, self.nb_nodes+1)
+        self.nodes_indices = np.arange(1, self.nb_nodes + 1)
         self.arity = np.max(np.sum(self.adjacency_matrix, 1))
-        self._parent = np.matmul(range(1, self.nb_nodes+1),
-                                 self.adjacency_matrix)
+        self._parent = np.matmul(range(1, self.nb_nodes + 1), self.adjacency_matrix)
         self.root = self.nodes_indices[self._parent == 0][0]
         self.is_leaf = np.repeat(False, self.nb_nodes)
-        self.is_leaf[self.dim2ind-1] = True
+        self.is_leaf[self.dim2ind - 1] = True
 
         _children = np.zeros([self.arity, self.nb_nodes], dtype=int)
         for i in range(self.nb_nodes):
             ind = np.nonzero(self.adjacency_matrix[i, :])[0]
-            _children[0:len(ind), i] = ind+1
+            _children[0: len(ind), i] = ind + 1
         self._children = _children
 
         _child_number = np.zeros(self.nb_nodes, dtype=int)
         for i in range(self.nb_nodes):
             if self._parent[i]:
-                ind1 = self._children[:, self._parent[i]-1]
-                ind2 = np.nonzero(ind1 == i+1)
+                ind1 = self._children[:, self._parent[i] - 1]
+                ind2 = np.nonzero(ind1 == i + 1)
                 if ind2[0].size:
-                    _child_number[i] = ind2[0][0]+1
+                    _child_number[i] = ind2[0][0] + 1
         self._child_number = _child_number
 
         _sibling = np.zeros([self.arity, self.nb_nodes], dtype=int)
         for i in range(self.nb_nodes):
             if self._parent[i]:
-                _sibling[:, i] = self._children[:, self._parent[i]-1]
+                _sibling[:, i] = self._children[:, self._parent[i] - 1]
         self.sibling = _sibling
 
         _level = np.zeros(self.nb_nodes, dtype=int)
-        ind = self._children[:, self.root-1]
+        ind = self._children[:, self.root - 1]
         ind = ind[ind != 0]
         level = 1
         while ind.size:
-            _level[ind-1] = level
-            ind = np.reshape(self._children[:, ind-1], [1, -1])
+            _level[ind - 1] = level
+            ind = np.reshape(self._children[:, ind - 1], [1, -1])
             ind = ind[np.nonzero(ind)]
             level += 1
         self.level = _level
 
         self.internal_nodes = np.setdiff1d(self.nodes_indices, self.dim2ind)
-        self.nodes_parent_of_leaves = np.unique(self._parent[self.dim2ind-1])
+        self.nodes_parent_of_leaves = np.unique(self._parent[self.dim2ind - 1])
         return self
 
     @staticmethod
     def trivial(order):
-        '''
+        """
         Create a dimension tree with one level.
 
         Parameters
@@ -610,15 +710,15 @@ class DimensionTree:
         DimensionTree
             Trivial dimension tree.
 
-        '''
-        d2i = np.arange(2, order+2)
-        adj_mat = np.zeros([order+1]*2, dtype=int)
+        """
+        d2i = np.arange(2, order + 2)
+        adj_mat = np.zeros([order + 1] * 2, dtype=int)
         adj_mat[0, 1:] = 1
         return DimensionTree(d2i, adj_mat)
 
     @staticmethod
     def linear(order):
-        '''
+        """
         Create a linear dimension tree.
 
         Parameters
@@ -631,7 +731,7 @@ class DimensionTree:
         DimensionTree
             Linear dimension tree.
 
-        '''
+        """
         order = np.array(order)
         if order.size == 1:
             dim = order
@@ -640,18 +740,18 @@ class DimensionTree:
             dim = order.size
             order -= 1
 
-        d2i = np.concatenate(([2*dim-2], 2*np.arange(dim-1, 0, -1)+1))
-        adj_mat = np.zeros([2*dim-1]*2, dtype=int)
+        d2i = np.concatenate(([2 * dim - 2], 2 * np.arange(dim - 1, 0, -1) + 1))
+        adj_mat = np.zeros([2 * dim - 1] * 2, dtype=int)
         adj_mat[0, 1:3] = 1
-        for level in range(dim-2):
-            adj_mat[2*level+1, 2*level+3] = 1
-            adj_mat[2*level+1, 2*level+4] = 1
+        for level in range(dim - 2):
+            adj_mat[2 * level + 1, 2 * level + 3] = 1
+            adj_mat[2 * level + 1, 2 * level + 4] = 1
         d2i[order] = d2i.flatten()
         return DimensionTree(d2i, adj_mat)
 
     @staticmethod
     def balanced(order):
-        '''
+        """
         Create a balanced dimension tree.
 
         Parameters
@@ -664,7 +764,7 @@ class DimensionTree:
         DimensionTree
             Balanced dimension tree.
 
-        '''
+        """
         order = np.array(order)
         if order.size == 1:
             dim = order
@@ -673,17 +773,17 @@ class DimensionTree:
             dim = order.size
             order -= 1
 
-        d2i = np.arange(dim, 2*dim)
-        adj_mat = np.zeros([2*dim-1]*2, dtype=int)
+        d2i = np.arange(dim, 2 * dim)
+        adj_mat = np.zeros([2 * dim - 1] * 2, dtype=int)
         adj_mat[0, 1:3] = 1
-        for i in range(1, dim-1):
-            adj_mat[i, 2*i+1:2*i+3] = 1
+        for i in range(1, dim - 1):
+            adj_mat[i, 2 * i + 1: 2 * i + 3] = 1
         d2i[order] = d2i.flatten()
         return DimensionTree(d2i, adj_mat)
 
     @staticmethod
     def random(order, arity=2):
-        '''
+        """
         Create a random dimension tree over {1,...,order}.
 
         If arity is an interval [amin,amin], then the number of children of a
@@ -702,7 +802,7 @@ class DimensionTree:
         DimensionTree
             Random dimension tree.
 
-        '''
+        """
         arity = np.array(arity)
         if arity.size == 1:
             arity = np.repeat(arity, 2)
@@ -710,31 +810,32 @@ class DimensionTree:
         dims_nodes = [np.arange(order)]
         nb_nodes = 1
         new_nodes = [1]
-        adj_mat = np.zeros([2*order]*2, dtype=int)
+        adj_mat = np.zeros([2 * order] * 2, dtype=int)
         dim2ind = np.zeros(order, dtype=int)
 
         while len(new_nodes) != 0:
             parent_nodes = new_nodes
             new_nodes = []
             for pnod in parent_nodes:
-                dims = dims_nodes[pnod-1]
+                dims = dims_nodes[pnod - 1]
                 if len(dims) == 1:
-                    dim2ind[dims-1] = pnod
+                    dim2ind[dims - 1] = pnod
                 else:
-                    pnod_arity = np.min(np.append(np.random.randint(*arity),
-                                                  dims.size))
+                    pnod_arity = np.min(np.append(np.random.randint(*arity), dims.size))
                     for k in range(pnod_arity):
                         nb_nodes += 1
                         new_nodes.append(nb_nodes)
-                        adj_mat[pnod-1, nb_nodes-1] = 1
-                        if k == pnod_arity-1:
+                        adj_mat[pnod - 1, nb_nodes - 1] = 1
+                        if k == pnod_arity - 1:
                             dims_nodes.append(dims)
                             dims = []
                         else:
-                            n_children = np.max(np.append(np.random.randint(
-                                len(dims)-pnod_arity+k+2), 1))
-                            perm = np.random.\
-                                permutation(len(dims))[:n_children]
+                            n_children = np.max(
+                                np.append(
+                                    np.random.randint(len(dims) - pnod_arity + k + 2), 1
+                                )
+                            )
+                            perm = np.random.permutation(len(dims))[:n_children]
                             dims_nodes.append(dims[perm])
                             dims = np.delete(dims, perm)
                         if len(dims) == 0:
